from typing import Optional

from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from flask import jsonify

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.exception.service_error_code_enum import ServiceErrorCode
from bot_api.exception.service_exception import ServiceException
from bot_api.filter.discord.server_select_criteria import ServerSelectCriteria
from bot_api.model.discord.server_dto import ServerDTO
from bot_api.model.discord.server_filtered_result_dto import ServerFilteredResultDTO
from bot_api.model.error_dto import ErrorDTO
from bot_api.transformer.server_transformer import ServerTransformer
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.auth_role_enum import AuthRoleEnum


class DiscordService:

    def __init__(
            self,
            bot: DiscordBotServiceABC,
            servers: ServerRepositoryABC,
            auth: AuthServiceABC,
            users: UserRepositoryABC,
    ):
        self._bot = bot
        self._servers = servers
        self._auth = auth
        self._users = users

    async def get_all_servers(self) -> List[ServerDTO]:
        servers = self._servers.get_servers()
        return servers.select(lambda x: ServerTransformer.to_dto(
            x,
            self._bot.get_guild(x.discord_server_id).name,
            self._bot.get_guild(x.discord_server_id).member_count,
            self._bot.get_guild(x.discord_server_id).icon
        ))

    async def get_all_servers_by_user(self) -> List[ServerDTO]:
        token = self._auth.get_decoded_token_from_request()
        if token is None or 'email' not in token or 'role' not in token:
            raise ServiceException(ServiceErrorCode.InvalidData, 'Token invalid')

        role = AuthRoleEnum(token['role'])
        if role == AuthRoleEnum.admin:
            servers = self._servers.get_servers()
        else:
            user = await self._auth.find_auth_user_by_email_async(token['email'])
            user_from_db = self._users.find_user_by_id(0 if user.user_id is None else user.user_id)
            servers = self._servers.get_servers().where(lambda x: user_from_db is not None and x.server_id == user_from_db.server.server_id)

        return servers.select(lambda x: ServerTransformer.to_dto(
            x,
            self._bot.get_guild(x.discord_server_id).name,
            self._bot.get_guild(x.discord_server_id).member_count,
            self._bot.get_guild(x.discord_server_id).icon
        ))

    async def get_filtered_servers_async(self, criteria: ServerSelectCriteria) -> ServerFilteredResultDTO:
        token = self._auth.get_decoded_token_from_request()
        if token is None or 'email' not in token or 'role' not in token:
            raise ServiceException(ServiceErrorCode.InvalidData, 'Token invalid')

        role = AuthRoleEnum(token['role'])
        filtered_result = self._servers.get_filtered_servers(criteria)
        servers = filtered_result.result
        if role != AuthRoleEnum.admin:
            user = await self._auth.find_auth_user_by_email_async(token['email'])
            user_from_db = self._users.find_user_by_id(0 if user.user_id is None else user.user_id)
            servers = servers.where(lambda x: user_from_db is not None and x.server_id == user_from_db.server.server_id)

        result = servers.select(
            lambda x: ServerTransformer.to_dto(
                x,
                self._bot.get_guild(x.discord_server_id).name,
                self._bot.get_guild(x.discord_server_id).member_count,
                self._bot.get_guild(x.discord_server_id).icon
            )
        )

        if criteria.name is not None and criteria.name != '':
            result = result.where(lambda x: criteria.name.lower() in x.name.lower() or x.name.lower() == criteria.name.lower())

        return ServerFilteredResultDTO(
            List(ServerDTO, result),
            filtered_result.total_count
        )
